@@ -4,14 +4,13 @@ import os
 import sys
 
 import logging
-logging.basicConfig()
 
 import unittest
 import urllib2
-from urlparse import urlparse
+from urlparse import urlparse, parse_qsl, parse_qs
 from urllib2 import Request
 
-from cgi import parse_qs
+logging.basicConfig()
 
 # prefer local copy to the one which is installed
 # hack from http://stackoverflow.com/a/6098238/280539
@@ -23,13 +22,6 @@ if _top_level_path not in sys.path:
     sys.path.insert(0, _top_level_path)
 # end of hack
 
-from SPARQLWrapper import SPARQLWrapper
-from SPARQLWrapper import XML, GET, POST, JSON, JSONLD, N3, TURTLE, RDF, SELECT, INSERT
-from SPARQLWrapper import URLENCODED, POSTDIRECTLY
-from SPARQLWrapper import BASIC, DIGEST
-from SPARQLWrapper.Wrapper import QueryResult, QueryBadFormed, EndPointNotFound, EndPointInternalError
-
-
 # we don't want to let Wrapper do real web-requests. so, we are…
 # constructing a simple Mock!
 from urllib2 import HTTPError
@@ -39,6 +31,12 @@ import warnings
 
 import SPARQLWrapper.Wrapper as _victim
 
+from SPARQLWrapper import SPARQLWrapper
+from SPARQLWrapper import XML, GET, POST, JSON, JSONLD, N3, TURTLE, RDF, SELECT, INSERT, RDFXML, CSV, TSV
+from SPARQLWrapper import URLENCODED, POSTDIRECTLY
+from SPARQLWrapper import BASIC, DIGEST
+from SPARQLWrapper.Wrapper import QueryResult, QueryBadFormed, EndPointNotFound, EndPointInternalError
+
 
 class FakeResult(object):
     def __init__(self, request):
@@ -120,7 +118,7 @@ class SPARQLWrapper_Test(TestCase):
             return parameters
         else:
             result = {}
-            for k,vs in parameters.iteritems():
+            for k, vs in parameters.iteritems():
                 result[k] = [v.encode('utf-8') for v in vs]
             return result
 
@@ -159,28 +157,37 @@ class SPARQLWrapper_Test(TestCase):
         self.wrapper.setQuery('CONSTRUCT WHERE {?a ?b ?c}')
         self.wrapper.setReturnFormat(N3)
         self.wrapper.addParameter('a', 'b')
+        self.wrapper.setOnlyConneg(True)
 
         request = self._get_request(self.wrapper)
         parameters = self._get_parameters_from_request(request)
+        onlyConneg = self.wrapper.onlyConneg
 
         self.assertEqual('POST', request.get_method())
         self.assertTrue(parameters['query'][0].startswith('CONSTRUCT'))
         self.assertTrue('rdf+n3' in request.get_header('Accept'))
         self.assertTrue('a' in parameters)
+        self.assertTrue(onlyConneg)
 
         self.wrapper.resetQuery()
 
         request = self._get_request(self.wrapper)
         parameters = self._get_parameters_from_request(request)
+        onlyConneg = self.wrapper.onlyConneg
 
         self.assertEqual('GET', request.get_method())
         self.assertTrue(parameters['query'][0].startswith('SELECT'))
         self.assertFalse('rdf+n3' in request.get_header('Accept'))
         self.assertTrue('sparql-results+xml' in request.get_header('Accept'))
         self.assertFalse('a' in parameters)
+        self.assertFalse('a' in parameters)
+        self.assertTrue(onlyConneg)
 
     def testSetReturnFormat(self):
-        self.assertRaises(ValueError, self.wrapper.setReturnFormat, 'nonexistent format')
+        with warnings.catch_warnings(record=True) as w:
+            self.wrapper.setReturnFormat('nonexistent format')
+            self.assertEqual(1, len(w), "Warning due to non expected format")
+
         self.assertEqual(XML, self.wrapper.query().requestedFormat)
 
         self.wrapper.setReturnFormat(JSON)
@@ -189,7 +196,7 @@ class SPARQLWrapper_Test(TestCase):
         try:
             import rdflib_jsonld
             self.wrapper.setReturnFormat(JSONLD)
-            self.assertEqual(JSONLD, self.wrapper.query().requestedFormat)   
+            self.assertEqual(JSONLD, self.wrapper.query().requestedFormat)
         except ImportError:
             self.assertRaises(ValueError, self.wrapper.setReturnFormat, JSONLD)
 
@@ -199,6 +206,9 @@ class SPARQLWrapper_Test(TestCase):
         self.assertTrue(self.wrapper.supportsReturnFormat(TURTLE))
         self.assertTrue(self.wrapper.supportsReturnFormat(N3))
         self.assertTrue(self.wrapper.supportsReturnFormat(RDF))
+        self.assertTrue(self.wrapper.supportsReturnFormat(RDFXML))
+        self.assertTrue(self.wrapper.supportsReturnFormat(CSV))
+        self.assertTrue(self.wrapper.supportsReturnFormat(TSV))
         self.assertFalse(self.wrapper.supportsReturnFormat('nonexistent format'))
 
         try:
@@ -229,11 +239,59 @@ class SPARQLWrapper_Test(TestCase):
         self.wrapper.setCredentials('login', 'password')
         request = self._get_request(self.wrapper)
         self.assertTrue(request.has_header('Authorization'))
-        
+
         # expected header for login:password
         # should succeed for python 3 since pull request #72
         self.assertEqual("Basic bG9naW46cGFzc3dvcmQ=", request.get_header('Authorization'))
 
+    def testAddCustomHttpHeader(self):
+        request = self._get_request(self.wrapper)
+        self.assertFalse(request.has_header('Foo'))
+
+        # Add new header field name
+        self.wrapper.addCustomHttpHeader('Foo', 'bar')
+        request = self._get_request(self.wrapper)
+        self.assertTrue(request.has_header('Foo'))
+        self.assertEqual("bar", request.get_header('Foo'))
+
+        # Override a new field name
+        self.wrapper.addCustomHttpHeader('Foo', 'bar')
+        request = self._get_request(self.wrapper)
+        self.assertTrue(request.has_header('Foo'))
+        self.assertEqual("bar", request.get_header('Foo'))
+        self.wrapper.addCustomHttpHeader('Foo', 'bar_2')
+        request = self._get_request(self.wrapper)
+        self.assertTrue(request.has_header('Foo'))
+        self.assertEqual("bar_2", request.get_header('Foo'))
+
+        # Override header field name
+        self.wrapper.addCustomHttpHeader('User-agent', 'Another UA')
+        request = self._get_request(self.wrapper)
+        self.assertEqual("Another UA", request.get_header('User-agent'))
+
+    def testClearCustomHttpHeader(self):
+        request = self._get_request(self.wrapper)
+        self.assertFalse(request.has_header('Foo'))
+
+        # Add new header field name
+        self.wrapper.addCustomHttpHeader('Foo_1', 'bar_1')
+        self.wrapper.addCustomHttpHeader('Foo_2', 'bar_2')
+        self.wrapper.addCustomHttpHeader('Foo_3', 'bar_3')
+
+
+        self.assertFalse(self.wrapper.clearCustomHttpHeader('Foo_4'))
+        self.assertTrue(self.wrapper.clearCustomHttpHeader('Foo_3'))
+
+        customHttpHeaders = self.wrapper.customHttpHeaders
+
+        self.assertTrue('Foo_1' in customHttpHeaders)
+        self.assertTrue('Foo_2' in customHttpHeaders)
+        self.assertEqual('bar_1', customHttpHeaders['Foo_1'])
+        self.assertEqual('bar_2', customHttpHeaders['Foo_2'])
+
+        self.assertFalse(self.wrapper.clearCustomHttpHeader('Foo_3'), 'already cleaned')
+
+
     def testSetHTTPAuth(self):
         self.assertRaises(TypeError, self.wrapper.setHTTPAuth, 123)
 
@@ -260,11 +318,11 @@ class SPARQLWrapper_Test(TestCase):
 
         self.wrapper.setQuery('PREFIX e: <http://example.org/> INSERT {e:a e:b e:c}')
         self.assertEqual(INSERT, self.wrapper.queryType)
-        
-        self.wrapper.setQuery("""#CONSTRUCT {?s ?p ?o} 
+
+        self.wrapper.setQuery("""#CONSTRUCT {?s ?p ?o}
                                    SELECT ?s ?p ?o
                                    WHERE {?s ?p ?o}""")
-        self.assertEqual(SELECT, self.wrapper.queryType)        
+        self.assertEqual(SELECT, self.wrapper.queryType)
 
         with warnings.catch_warnings(record=True) as w:
             self.wrapper.setQuery('UNKNOWN {e:a e:b e:c}')
@@ -489,6 +547,161 @@ class SPARQLWrapper_Test(TestCase):
         finally:
             _victim.QueryResult = _oldQueryResult
 
+    def testComments(self):
+        # see issue #32
+        self.wrapper.setQuery("""
+# this is a comment
+select * where { ?s ?p ?o }
+""")
+        self.assertTrue(self.wrapper.isSparqlQueryRequest())
+
+    def testHashInPrefixes(self):
+        # see issue #77
+        self.wrapper.setQuery("""
+PREFIX rdfs:    <http://www.w3.org/2000/01/rdf-schema#>
+select * where { ?s ?p ?o }
+""")
+        self.assertTrue(self.wrapper.isSparqlQueryRequest())
+
+    def testHashInPrefixComplex(self):
+        # see issue #77
+        self.wrapper.setQuery("""
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+PREFIX rdfs:    <http://www.w3.org/2000/01/rdf-schema#>
+PREFIX weather: <http://hal.zamia.org/weather/>
+PREFIX dbo:     <http://dbpedia.org/ontology/> 
+PREFIX dbr:     <http://dbpedia.org/resource/> 
+PREFIX dbp:     <http://dbpedia.org/property/> 
+PREFIX xml:     <http://www.w3.org/XML/1998/namespace> 
+PREFIX xsd:     <http://www.w3.org/2001/XMLSchema#> 
+
+SELECT DISTINCT ?location ?cityid ?timezone ?label
+WHERE {
+  ?location weather:cityid ?cityid .
+  ?location weather:timezone ?timezone .
+  ?location rdfs:label ?label .
+}
+""")
+        self.assertTrue(self.wrapper.isSparqlQueryRequest())
+
+    def testHashWithNoComments(self):
+        # see issue #77
+        query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+
+SELECT *
+WHERE {
+  ?s ?p ?o .
+}
+"""
+        parsed_query = self.wrapper._cleanComments(query)
+        self.assertEquals(query, parsed_query)
+
+    def testCommentBeginningLine(self):
+        # see issue #77
+        query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+# a comment
+SELECT *
+WHERE {
+  ?s ?p ?o .
+}
+"""
+        expected_parsed_query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+
+SELECT *
+WHERE {
+  ?s ?p ?o .
+}
+"""
+        parsed_query = self.wrapper._cleanComments(query)
+        self.assertEquals(expected_parsed_query, parsed_query)
+
+    def testCommentEmtpyLine(self):
+        # see issue #77
+        query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+     # a comment
+SELECT *
+WHERE {
+  ?s ?p ?o .
+}
+"""
+        expected_parsed_query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+
+SELECT *
+WHERE {
+  ?s ?p ?o .
+}
+"""
+        parsed_query = self.wrapper._cleanComments(query)
+        self.assertEquals(expected_parsed_query, parsed_query)
+
+    def testCommentsFirstLine(self):
+        # see issue #77
+        query = """#CONSTRUCT {?s ?p ?o}
+                                   SELECT ?s ?p ?o
+                                   WHERE {?s ?p ?o}"""
+        expected_parsed_query = """
+
+                                   SELECT ?s ?p ?o
+                                   WHERE {?s ?p ?o}"""
+
+        parsed_query = self.wrapper._cleanComments(query)
+        self.assertEquals(expected_parsed_query, parsed_query)
+
+    @unittest.skip("issue #80")
+    def testCommentAfterStatements(self):
+        # see issue #77
+        query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+
+SELECT *
+WHERE {     # this is the where condition
+  ?s ?p ?o .
+}
+"""
+        expected_parsed_query = """
+PREFIX rdf:     <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+
+SELECT *
+WHERE {
+  ?s ?p ?o .
+}
+"""
+        parsed_query = self.wrapper._cleanComments(query)
+        self.assertEquals(expected_parsed_query, parsed_query)
+
+    def testSingleLineQueryLine(self):
+        # see issue #74
+        query = "prefix whatever: <http://example.org/blah#> ASK { ?s ?p ?o }"
+        parsed_query = self.wrapper._cleanComments(query)
+        self.assertEquals(query, parsed_query)
+
+        self.wrapper.setQuery(query)
+        self.assertTrue(self.wrapper.isSparqlQueryRequest())
+
+    def testOnlyConneg(self):
+        # see issue #82
+        query = "prefix whatever: <http://example.org/blah#> ASK { ?s ?p ?o }"
+        self.wrapper.setOnlyConneg(False)
+        self.wrapper.setQuery(query)
+        request = self._get_request(self.wrapper)
+        request_params = dict(parse_qsl(urlparse(request.get_full_url()).query))
+        for returnFormatSetting in ["format", "output", "results"]: # Obviously _returnFormatSetting is not accessible from SPARQLWrapper, so we copy&paste the possible values
+            self.assertTrue(returnFormatSetting in request_params, "URL parameter '%s' was not sent, and it was expected" %returnFormatSetting)
+
+        #ONLY Content Negotiation
+        self.wrapper.resetQuery()
+        self.wrapper.setOnlyConneg(True)
+        self.wrapper.setQuery(query)
+        request = self._get_request(self.wrapper)
+        request_params = dict(parse_qsl(urlparse(request.get_full_url()).query))
+        for returnFormatSetting in ["format", "output", "results"]: # Obviously _returnFormatSetting is not accessible from SPARQLWrapper, so we copy&paste the possible values
+            self.assertFalse(returnFormatSetting in request_params, "URL parameter '%s' was sent, and it was not expected (only Content Negotiation)" %returnFormatSetting)
+
 
 class QueryResult_Test(unittest.TestCase):
 
@@ -554,8 +767,8 @@ class QueryResult_Test(unittest.TestCase):
 
         def _mime_vs_type(mime, requested_type):
             """
-            @param mime:
-            @param requested_type:
+            @param mime: mimetype/Content-Type of the response
+            @param requested_type: requested mimetype (alias)
             @return: number of warnings produced by combo
             """
             with warnings.catch_warnings(record=True) as w:
@@ -568,14 +781,22 @@ class QueryResult_Test(unittest.TestCase):
 
                 return len(w)
 
+        # In the cases of "application/ld+json" and "application/rdf+xml", the
+        # RDFLib raised a warning because the manually created QueryResult has no real
+        # response value (implemented a fake read).
+        # "WARNING:rdflib.term:  does not look like a valid URI, trying to serialize this will break."
         self.assertEqual(0, _mime_vs_type("application/sparql-results+xml", XML))
         self.assertEqual(0, _mime_vs_type("application/sparql-results+json", JSON))
         self.assertEqual(0, _mime_vs_type("text/n3", N3))
         self.assertEqual(0, _mime_vs_type("text/turtle", TURTLE))
-        self.assertEqual(0, _mime_vs_type("application/ld+json", JSON))
-        self.assertEqual(0, _mime_vs_type("application/ld+json", JSONLD))
-        self.assertEqual(0, _mime_vs_type("application/rdf+xml", XML))
-        self.assertEqual(0, _mime_vs_type("application/rdf+xml", RDF))
+        self.assertEqual(0, _mime_vs_type("application/ld+json", JSON)) # Warning
+        self.assertEqual(0, _mime_vs_type("application/ld+json", JSONLD)) # Warning
+        self.assertEqual(0, _mime_vs_type("application/rdf+xml", XML)) # Warning
+        self.assertEqual(0, _mime_vs_type("application/rdf+xml", RDF)) # Warning
+        self.assertEqual(0, _mime_vs_type("application/rdf+xml", RDFXML)) # Warning
+        self.assertEqual(0, _mime_vs_type("text/csv", CSV))
+        self.assertEqual(0, _mime_vs_type("text/tab-separated-values", TSV))
+        self.assertEqual(0, _mime_vs_type("application/xml", XML))
 
         self.assertEqual(1, _mime_vs_type("application/x-foo-bar", XML), "invalid mime")
 
@@ -583,11 +804,10 @@ class QueryResult_Test(unittest.TestCase):
         self.assertEqual(1, _mime_vs_type("application/sparql-results+json", XML))
         self.assertEqual(1, _mime_vs_type("text/n3", JSON))
         self.assertEqual(1, _mime_vs_type("text/turtle", XML))
-        self.assertEqual(1, _mime_vs_type("application/ld+json", XML))
-        self.assertEqual(1, _mime_vs_type("application/ld+json", N3))
-        self.assertEqual(1, _mime_vs_type("application/rdf+xml", JSON))
-        self.assertEqual(1, _mime_vs_type("application/rdf+xml", N3))
-
+        self.assertEqual(1, _mime_vs_type("application/ld+json", XML))  # Warning
+        self.assertEqual(1, _mime_vs_type("application/ld+json", N3))  # Warning
+        self.assertEqual(1, _mime_vs_type("application/rdf+xml", JSON))  # Warning
+        self.assertEqual(1, _mime_vs_type("application/rdf+xml", N3))  # Warning
 
 if __name__ == "__main__":
     unittest.main()
